==> pyrefly-0.17.0.tar.gz: pyrefly-0.17.0/pyrefly/lib/test/lsp/definition.rs <==
/*
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

use pretty_assertions::assert_eq;
use ruff_text_size::TextSize;

use crate::module::module_info::TextRangeWithModuleInfo;
use crate::state::handle::Handle;
use crate::state::state::State;
use crate::test::util::code_frame_of_source_at_range;
use crate::test::util::get_batched_lsp_operations_report;
use crate::test::util::get_batched_lsp_operations_report_allow_error;

fn get_test_report(state: &State, handle: &Handle, position: TextSize) -> String {
    if let Some(TextRangeWithModuleInfo { module_info, range }) =
        state.transaction().goto_definition(handle, position)
    {
        format!(
            "Definition Result:\n{}",
            code_frame_of_source_at_range(module_info.contents(), range)
        )
    } else {
        "Definition Result: None".to_owned()
    }
}

#[test]
fn ignored_test() {
    let code = r#"
x = 1 # go-to-definition is unsupported for literals
#   ^
"#;
    let report = get_batched_lsp_operations_report(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
2 | x = 1 # go-to-definition is unsupported for literals
        ^
Definition Result: None
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn basic_test() {
    let code = r#"
from typing import Literal

def f(x: list[int], y: str, z: Literal[42]):
#   ^                           ^
    return x

yyy = f([1, 2, 3], "test", 42)
#     ^

class A: pass
class B(A): pass
#       ^
"#;
    let report = get_batched_lsp_operations_report(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
4 | def f(x: list[int], y: str, z: Literal[42]):
        ^
Definition Result:
4 | def f(x: list[int], y: str, z: Literal[42]):
        ^

4 | def f(x: list[int], y: str, z: Literal[42]):
                                    ^
Definition Result:
250 | Literal: _SpecialForm
      ^^^^^^^

8 | yyy = f([1, 2, 3], "test", 42)
          ^
Definition Result:
4 | def f(x: list[int], y: str, z: Literal[42]):
        ^

12 | class B(A): pass
             ^
Definition Result:
11 | class A: pass
           ^
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn named_import_tests() {
    let code_import_provider: &str = r#"
from typing import Literal

def f(x: list[int], y: str, z: Literal[42]):
    return x
"#;
    let code_test: &str = r#"
from typing import Literal
from .import_provider import f
#           ^                ^

foo: Literal[1] = 1
#        ^
bar = f([1], "", 42)
#     ^
"#;

    let report = get_batched_lsp_operations_report(
        &[
            ("main", code_test),
            ("import_provider", code_import_provider),
        ],
        get_test_report,
    );
    assert_eq!(
        r#"
# main.py
3 | from .import_provider import f
                ^
Definition Result:
1 | 
    ^

3 | from .import_provider import f
                                 ^
Definition Result:
4 | def f(x: list[int], y: str, z: Literal[42]):
        ^

6 | foo: Literal[1] = 1
             ^
Definition Result:
250 | Literal: _SpecialForm
      ^^^^^^^

8 | bar = f([1], "", 42)
          ^
Definition Result:
4 | def f(x: list[int], y: str, z: Literal[42]):
        ^


# import_provider.py
"#
        .trim(),
        report.trim()
    );
}

#[test]
fn aliased_import_tests() {
    let code_import_provider: &str = r#"
# top of module
class Foo: pass
    "#;
    let code_test: &str = r#"
from import_provider import Foo as F
#                                  ^
import import_provider as ip
#                         ^

def f(x: ip.Foo, y: F):
#        ^          ^
    return x
"#;

    let report = get_batched_lsp_operations_report(
        &[
            ("main", code_test),
            ("import_provider", code_import_provider),
        ],
        get_test_report,
    );
    assert_eq!(
        r#"
# main.py
2 | from import_provider import Foo as F
                                       ^
Definition Result:
3 | class Foo: pass
          ^^^

4 | import import_provider as ip
                              ^
Definition Result:
1 | 
    ^

7 | def f(x: ip.Foo, y: F):
             ^
Definition Result:
1 | 
    ^

7 | def f(x: ip.Foo, y: F):
                        ^
Definition Result:
3 | class Foo: pass
          ^^^


# import_provider.py
"#
        .trim(),
        report.trim()
    );
}

#[test]
fn incorrect_import_tests() {
    let code_test: &str = r#"
from .....import_provider import baz
baz
# ^
"#;
    let report =
        get_batched_lsp_operations_report_allow_error(&[("main", code_test)], get_test_report);
    assert_eq!(
        r#"
# main.py
3 | baz
      ^
Definition Result:
2 | from .....import_provider import baz
                                     ^^^
"#
        .trim(),
        report.trim()
    )
}

#[test]
fn star_import_tests() {
    let code_import_provider: &str = r#"
def f():
    pass
"#;
    let code_test: &str = r#"
from .import_provider import *

bar = f() # should jump to definition in import_provider
#     ^
"#;

    let report = get_batched_lsp_operations_report(
        &[
            ("main", code_test),
            ("import_provider", code_import_provider),
        ],
        get_test_report,
    );
    assert_eq!(
        r#"
# main.py
4 | bar = f() # should jump to definition in import_provider
          ^
Definition Result:
2 | def f():
        ^


# import_provider.py
"#
        .trim(),
        report.trim()
    );
}

#[test]
fn inline_import_test() {
    let code_import_provider: &str = r#"
def f():
        pass"#;
    let code_test: &str = r#"
def foo() -> None:
    from .import_provider import f
    bar = f()
    #     ^
    "#;
    let report = get_batched_lsp_operations_report(
        &[
            ("main", code_test),
            ("import_provider", code_import_provider),
        ],
        get_test_report,
    );
    assert_eq!(
        r#"
# main.py
4 |     bar = f()
              ^
Definition Result:
2 | def f():
        ^


# import_provider.py
"#
        .trim(),
        report.trim()
    );
}

#[test]
fn import_of_import_test() {
    let code_import_provider2: &str = r#"
def f():
        pass"#;
    let code_import_provider: &str = r#"
from .import_provider2 import f
"#;
    let code_test: &str = r#"
from .import_provider import *

bar = f() # should jump to definition in import_provider2
#     ^
    "#;
    let report = get_batched_lsp_operations_report(
        &[
            ("main", code_test),
            ("import_provider", code_import_provider),
            ("import_provider2", code_import_provider2),
        ],
        get_test_report,
    );
    assert_eq!(
        r#"
# main.py
4 | bar = f() # should jump to definition in import_provider2
          ^
Definition Result:
2 | def f():
        ^


# import_provider.py

# import_provider2.py
"#
        .trim(),
        report.trim()
    );
}

#[test]
fn unresolved_named_import_test() {
    let code: &str = r#"
from .unresolved_import import f

bar = f()
#     ^
"#;

    let report = get_batched_lsp_operations_report_allow_error(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
4 | bar = f()
          ^
Definition Result:
2 | from .unresolved_import import f
                                   ^
"#
        .trim(),
        report.trim()
    );
}

#[test]
fn unresolved_star_import_test() {
    let code: &str = r#"
from .unresolved_import import *

bar = f()
#     ^
"#;

    let report = get_batched_lsp_operations_report_allow_error(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
4 | bar = f()
          ^
Definition Result: None

"#
        .trim(),
        report.trim()
    );
}

#[test]
fn goto_def_dead_code() {
    let code: &str = r#"
if False:
    x
#   ^
"#;

    let report = get_batched_lsp_operations_report_allow_error(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
3 |     x
        ^
Definition Result: None
"#
        .trim(),
        report.trim()
    );
}

#[test]
fn multi_definition_test() {
    let code = r#"
if True:
    xxxx = 1
else:
    xxxx = 2
xxxx # it's reasonable to only return the first def, but also reasonable to return both defs
# ^
"#;
    let report = get_batched_lsp_operations_report(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
6 | xxxx # it's reasonable to only return the first def, but also reasonable to return both defs
      ^
Definition Result:
3 |     xxxx = 1
        ^^^^
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn untyped_param_test() {
    let code = r#"
def f(untyped):
  untyped
#    ^
"#;
    let report = get_batched_lsp_operations_report(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
3 |   untyped
         ^
Definition Result:
2 | def f(untyped):
          ^^^^^^^
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn global_reference_test() {
    let code = r#"
x = 3
def f(untyped):
  global x
  x
# ^
"#;
    let report = get_batched_lsp_operations_report_allow_error(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
5 |   x
      ^
Definition Result:
2 | x = 3
    ^
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn reassigned_test() {
    let code = r#"
xxxx = 1
xxxx = 2
xxxx # should jump to the most recent definition
# ^
"#;
    let report = get_batched_lsp_operations_report(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
4 | xxxx # should jump to the most recent definition
      ^
Definition Result:
3 | xxxx = 2
    ^^^^
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn generics_test() {
    let code = r#"
def f[T](input: T):
#     ^         ^
  pass
"#;
    let report = get_batched_lsp_operations_report(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
2 | def f[T](input: T):
          ^
Definition Result:
2 | def f[T](input: T):
          ^

2 | def f[T](input: T):
                    ^
Definition Result:
2 | def f[T](input: T):
          ^
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn property_test() {
    let code = r#"
class MyClass:
  x = 5
c1 = MyClass()
c1.x
#  ^

class MyClassWithImplicitField:
  def __init__(self, name: str):
    self.name = name

c2 = MyClassWithImplicitField("")
c2.name
#  ^

class ExtendsMyClass(MyClass):
  y = 6
c3 = ExtendsMyClass()
c3.x
#  ^
c3.y
#  ^

class Union1:
  x = 5

class Union2:
  x = 6

c4: Union1 | Union2 = Union1()
c4.x
#  ^

dict = {"foo": '', "bar": 3}
dict["foo"]
#      ^
dict["bar"]
#      ^
"#;
    // TODO: property go-to-definition is unsupported
    let report = get_batched_lsp_operations_report(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
5 | c1.x
       ^
Definition Result:
3 |   x = 5
      ^

13 | c2.name
        ^
Definition Result:
10 |     self.name = name
              ^^^^

19 | c3.x
        ^
Definition Result:
3 |   x = 5
      ^

21 | c3.y
        ^
Definition Result:
17 |   y = 6
       ^

31 | c4.x
        ^
Definition Result: None

35 | dict["foo"]
            ^
Definition Result: None

37 | dict["bar"]
            ^
Definition Result: None
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn cross_module_property_test() {
    let code_class_provider = r#"
class MyClass:
  x = 5
"#;
    let code = r#"
from .my_class import MyClass
c1 = MyClass()
c1.x
#  ^
"#;
    let report = get_batched_lsp_operations_report(
        &[("main", code), ("my_class", code_class_provider)],
        get_test_report,
    );
    assert_eq!(
        r#"
# main.py
4 | c1.x
       ^
Definition Result:
3 |   x = 5
      ^


# my_class.py
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn module_attribute_test() {
    let code_class_provider = r#"
class MyClass:
  x = 5
"#;
    let code = r#"
import my_class
my_class.MyClass
#        ^
"#;
    let report = get_batched_lsp_operations_report(
        &[("main", code), ("my_class", code_class_provider)],
        get_test_report,
    );
    assert_eq!(
        r#"
# main.py
3 | my_class.MyClass
             ^
Definition Result:
2 | class MyClass:
          ^^^^^^^


# my_class.py
"#
        .trim(),
        report.trim(),
    );
}

==> pyrefly-0.17.1.tar.gz: pyrefly-0.17.1/pyrefly/lib/test/lsp/definition.rs <==
/*
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

use pretty_assertions::assert_eq;
use ruff_text_size::TextSize;

use crate::module::module_info::TextRangeWithModuleInfo;
use crate::state::handle::Handle;
use crate::state::state::State;
use crate::test::util::code_frame_of_source_at_range;
use crate::test::util::get_batched_lsp_operations_report;
use crate::test::util::get_batched_lsp_operations_report_allow_error;

fn get_test_report(state: &State, handle: &Handle, position: TextSize) -> String {
    if let Some(TextRangeWithModuleInfo { module_info, range }) =
        state.transaction().goto_definition(handle, position)
    {
        format!(
            "Definition Result:\n{}",
            code_frame_of_source_at_range(module_info.contents(), range)
        )
    } else {
        "Definition Result: None".to_owned()
    }
}

#[test]
fn ignored_test() {
    let code = r#"
x = 1 # go-to-definition is unsupported for literals
#   ^
"#;
    let report = get_batched_lsp_operations_report(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
2 | x = 1 # go-to-definition is unsupported for literals
        ^
Definition Result: None
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn basic_test() {
    let code = r#"
from typing import Literal

def f(x: list[int], y: str, z: Literal[42]):
#   ^                           ^
    return x

yyy = f([1, 2, 3], "test", 42)
#     ^

class A: pass
class B(A): pass
#       ^
"#;
    let report = get_batched_lsp_operations_report(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
4 | def f(x: list[int], y: str, z: Literal[42]):
        ^
Definition Result:
4 | def f(x: list[int], y: str, z: Literal[42]):
        ^

4 | def f(x: list[int], y: str, z: Literal[42]):
                                    ^
Definition Result:
250 | Literal: _SpecialForm
      ^^^^^^^

8 | yyy = f([1, 2, 3], "test", 42)
          ^
Definition Result:
4 | def f(x: list[int], y: str, z: Literal[42]):
        ^

12 | class B(A): pass
             ^
Definition Result:
11 | class A: pass
           ^
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn narrow_test() {
    let code = r#"
def f(x: int | None) -> int:
    if x is None:
        raise ValueError("x is None")
    return 0 if x else 1
#               ^
"#;
    let report = get_batched_lsp_operations_report(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
5 |     return 0 if x else 1
                    ^
Definition Result:
2 | def f(x: int | None) -> int:
          ^
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn named_import_tests() {
    let code_import_provider: &str = r#"
from typing import Literal

def f(x: list[int], y: str, z: Literal[42]):
    return x
"#;
    let code_test: &str = r#"
from typing import Literal
from .import_provider import f
# ^         ^                ^

foo: Literal[1] = 1
#        ^
bar = f([1], "", 42)
#     ^
"#;

    let report = get_batched_lsp_operations_report(
        &[
            ("main", code_test),
            ("import_provider", code_import_provider),
        ],
        get_test_report,
    );
    assert_eq!(
        r#"
# main.py
3 | from .import_provider import f
      ^
Definition Result: None

3 | from .import_provider import f
                ^
Definition Result:
1 | 
    ^

3 | from .import_provider import f
                                 ^
Definition Result:
4 | def f(x: list[int], y: str, z: Literal[42]):
        ^

6 | foo: Literal[1] = 1
             ^
Definition Result:
250 | Literal: _SpecialForm
      ^^^^^^^

8 | bar = f([1], "", 42)
          ^
Definition Result:
4 | def f(x: list[int], y: str, z: Literal[42]):
        ^


# import_provider.py
"#
        .trim(),
        report.trim()
    );
}

#[test]
fn aliased_import_tests() {
    let code_import_provider: &str = r#"
# top of module
class Foo: pass
    "#;
    let code_test: &str = r#"
from import_provider import Foo as F
#                                  ^
import import_provider as ip
#                         ^

def f(x: ip.Foo, y: F):
#        ^          ^
    return x
"#;

    let report = get_batched_lsp_operations_report(
        &[
            ("main", code_test),
            ("import_provider", code_import_provider),
        ],
        get_test_report,
    );
    assert_eq!(
        r#"
# main.py
2 | from import_provider import Foo as F
                                       ^
Definition Result:
3 | class Foo: pass
          ^^^

4 | import import_provider as ip
                              ^
Definition Result:
1 | 
    ^

7 | def f(x: ip.Foo, y: F):
             ^
Definition Result:
1 | 
    ^

7 | def f(x: ip.Foo, y: F):
                        ^
Definition Result:
3 | class Foo: pass
          ^^^


# import_provider.py
"#
        .trim(),
        report.trim()
    );
}

#[test]
fn incorrect_import_tests() {
    let code_test: &str = r#"
from .....import_provider import baz
baz
# ^
"#;
    let report =
        get_batched_lsp_operations_report_allow_error(&[("main", code_test)], get_test_report);
    assert_eq!(
        r#"
# main.py
3 | baz
      ^
Definition Result:
2 | from .....import_provider import baz
                                     ^^^
"#
        .trim(),
        report.trim()
    )
}

#[test]
fn star_import_tests() {
    let code_import_provider: &str = r#"
def f():
    pass
"#;
    let code_test: &str = r#"
from .import_provider import *

bar = f() # should jump to definition in import_provider
#     ^
"#;

    let report = get_batched_lsp_operations_report(
        &[
            ("main", code_test),
            ("import_provider", code_import_provider),
        ],
        get_test_report,
    );
    assert_eq!(
        r#"
# main.py
4 | bar = f() # should jump to definition in import_provider
          ^
Definition Result:
2 | def f():
        ^


# import_provider.py
"#
        .trim(),
        report.trim()
    );
}

#[test]
fn inline_import_test() {
    let code_import_provider: &str = r#"
def f():
        pass"#;
    let code_test: &str = r#"
def foo() -> None:
    from .import_provider import f
    bar = f()
    #     ^
    "#;
    let report = get_batched_lsp_operations_report(
        &[
            ("main", code_test),
            ("import_provider", code_import_provider),
        ],
        get_test_report,
    );
    assert_eq!(
        r#"
# main.py
4 |     bar = f()
              ^
Definition Result:
2 | def f():
        ^


# import_provider.py
"#
        .trim(),
        report.trim()
    );
}

#[test]
fn import_of_import_test() {
    let code_import_provider2: &str = r#"
def f():
        pass"#;
    let code_import_provider: &str = r#"
from .import_provider2 import f
"#;
    let code_test: &str = r#"
from .import_provider import *

bar = f() # should jump to definition in import_provider2
#     ^
    "#;
    let report = get_batched_lsp_operations_report(
        &[
            ("main", code_test),
            ("import_provider", code_import_provider),
            ("import_provider2", code_import_provider2),
        ],
        get_test_report,
    );
    assert_eq!(
        r#"
# main.py
4 | bar = f() # should jump to definition in import_provider2
          ^
Definition Result:
2 | def f():
        ^


# import_provider.py

# import_provider2.py
"#
        .trim(),
        report.trim()
    );
}

#[test]
fn unresolved_named_import_test() {
    let code: &str = r#"
from .unresolved_import import f

bar = f()
#     ^
"#;

    let report = get_batched_lsp_operations_report_allow_error(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
4 | bar = f()
          ^
Definition Result:
2 | from .unresolved_import import f
                                   ^
"#
        .trim(),
        report.trim()
    );
}

#[test]
fn unresolved_star_import_test() {
    let code: &str = r#"
from .unresolved_import import *

bar = f()
#     ^
"#;

    let report = get_batched_lsp_operations_report_allow_error(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
4 | bar = f()
          ^
Definition Result: None

"#
        .trim(),
        report.trim()
    );
}

#[test]
fn goto_def_dead_code() {
    let code: &str = r#"
if False:
    x
#   ^
"#;

    let report = get_batched_lsp_operations_report_allow_error(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
3 |     x
        ^
Definition Result: None
"#
        .trim(),
        report.trim()
    );
}

#[test]
fn multi_definition_test() {
    let code = r#"
if True:
    xxxx = 1
else:
    xxxx = 2
xxxx # it's reasonable to only return the first def, but also reasonable to return both defs
# ^
"#;
    let report = get_batched_lsp_operations_report(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
6 | xxxx # it's reasonable to only return the first def, but also reasonable to return both defs
      ^
Definition Result:
3 |     xxxx = 1
        ^^^^
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn untyped_param_test() {
    let code = r#"
def f(untyped):
  untyped
#    ^
"#;
    let report = get_batched_lsp_operations_report(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
3 |   untyped
         ^
Definition Result:
2 | def f(untyped):
          ^^^^^^^
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn global_reference_test() {
    let code = r#"
x = 3
def f(untyped):
  global x
  x
# ^
"#;
    let report = get_batched_lsp_operations_report_allow_error(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
5 |   x
      ^
Definition Result:
2 | x = 3
    ^
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn reassigned_test() {
    let code = r#"
xxxx = 1
xxxx = 2
xxxx # should jump to the most recent definition
# ^
"#;
    let report = get_batched_lsp_operations_report(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
4 | xxxx # should jump to the most recent definition
      ^
Definition Result:
3 | xxxx = 2
    ^^^^
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn generics_test() {
    let code = r#"
def f[T](input: T):
#     ^         ^
  pass
"#;
    let report = get_batched_lsp_operations_report(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
2 | def f[T](input: T):
          ^
Definition Result:
2 | def f[T](input: T):
          ^

2 | def f[T](input: T):
                    ^
Definition Result:
2 | def f[T](input: T):
          ^
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn property_test() {
    let code = r#"
class MyClass:
  x = 5
c1 = MyClass()
c1.x
#  ^

class MyClassWithImplicitField:
  def __init__(self, name: str):
    self.name = name

c2 = MyClassWithImplicitField("")
c2.name
#  ^

class ExtendsMyClass(MyClass):
  y = 6
c3 = ExtendsMyClass()
c3.x
#  ^
c3.y
#  ^

class Union1:
  x = 5

class Union2:
  x = 6

c4: Union1 | Union2 = Union1()
c4.x
#  ^

dict = {"foo": '', "bar": 3}
dict["foo"]
#      ^
dict["bar"]
#      ^
"#;
    // TODO: property go-to-definition is unsupported
    let report = get_batched_lsp_operations_report(&[("main", code)], get_test_report);
    assert_eq!(
        r#"
# main.py
5 | c1.x
       ^
Definition Result:
3 |   x = 5
      ^

13 | c2.name
        ^
Definition Result:
10 |     self.name = name
              ^^^^

19 | c3.x
        ^
Definition Result:
3 |   x = 5
      ^

21 | c3.y
        ^
Definition Result:
17 |   y = 6
       ^

31 | c4.x
        ^
Definition Result: None

35 | dict["foo"]
            ^
Definition Result: None

37 | dict["bar"]
            ^
Definition Result: None
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn cross_module_property_test() {
    let code_class_provider = r#"
class MyClass:
  x = 5
"#;
    let code = r#"
from .my_class import MyClass
c1 = MyClass()
c1.x
#  ^
"#;
    let report = get_batched_lsp_operations_report(
        &[("main", code), ("my_class", code_class_provider)],
        get_test_report,
    );
    assert_eq!(
        r#"
# main.py
4 | c1.x
       ^
Definition Result:
3 |   x = 5
      ^


# my_class.py
"#
        .trim(),
        report.trim(),
    );
}

#[test]
fn module_attribute_test() {
    let code_class_provider = r#"
class MyClass:
  x = 5
"#;
    let code = r#"
import my_class
my_class.MyClass
#        ^
"#;
    let report = get_batched_lsp_operations_report(
        &[("main", code), ("my_class", code_class_provider)],
        get_test_report,
    );
    assert_eq!(
        r#"
# main.py
3 | my_class.MyClass
             ^
Definition Result:
2 | class MyClass:
          ^^^^^^^


# my_class.py
"#
        .trim(),
        report.trim(),
    );
}
